-- Little People Crowd Simulation
-- Touch the screen to make the crowd rush to that point.
-- When no touch is active, each little person wanders around.

local people = {}      -- table to store our little people
local numPeople = 200  -- adjust this to simulate more or fewer people
local touchPos = nil   -- current touch position (nil when no touch)

function setup()
    -- Initialize each little person with a random starting position and zero velocity.
    for i = 1, numPeople do
        table.insert(people, {
            x = math.random(WIDTH),
            y = math.random(HEIGHT),
            vx = 0,
            vy = 0
        })
    end
end

function draw()
    background(40, 40, 50)
    
    -- Update and draw each little person.
    for i, p in ipairs(people) do
        if touchPos then
            -- Calculate the vector from the person to the touch position.
            local dx = touchPos.x - p.x
            local dy = touchPos.y - p.y
            local dist = math.sqrt(dx * dx + dy * dy)
            if dist > 1 then
                -- Accelerate toward the touch point.
                local accel = 500  -- acceleration (pixels per second^2)
                local ax = (dx / dist) * accel
                local ay = (dy / dist) * accel
                p.vx = p.vx + ax * DeltaTime
                p.vy = p.vy + ay * DeltaTime
            end
        else
            -- Apply a small random acceleration for wandering.
            local wanderAccel = 100
            local ax = (math.random() - 0.5) * wanderAccel
            local ay = (math.random() - 0.5) * wanderAccel
            p.vx = p.vx + ax * DeltaTime
            p.vy = p.vy + ay * DeltaTime
        end
        
        -- Apply damping to avoid runaway speeds.
        p.vx = p.vx * 0.95
        p.vy = p.vy * 0.95
        
        -- Update the position using the current velocity.
        p.x = p.x + p.vx * DeltaTime
        p.y = p.y + p.vy * DeltaTime
        
        -- Draw the little person as a circle.
        fill(255)
        ellipse(p.x, p.y, 10)
    end
end

function touched(touch)
    -- When the user touches (or moves a finger on) the screen, record that location.
    if touch.state == BEGAN or touch.state == MOVING then
        touchPos = { x = touch.x, y = touch.y }
    else
        touchPos = nil
    end
end